Module ic

Module ic 

Available on crate feature std only.
Integer counter (IC) based tokens

This module provides a token type that is generated from a global 128-bit counter with thread-local pools to optimize for throughput. The runtime memory consumption is proportional to the maximum number of threads that existed simultaneously because the memory allocated to hold a freelist is never released.

Structs

IcToken
A counter-based unforgeable token used to access the contents of a TokenLock.
IcTokenId
Token that cannot be used to access the contents of a TokenLock, but can be used to create a new TokenLock.
IcTokenUnsyncRef
Represents a borrow of IcToken constrained to a single thread.

Type Aliases

IcPinTokenLock
A pinned mutual exclusive primitive that can be accessed by presenting the matching IcToken.
IcTokenLock
A mutual exclusive primitive that can be accessed by presenting the matching IcToken.
UnsyncIcPinTokenLock
Like IcPinTokenLock but requires presenting IcTokenUnsyncRef, which is Unsync. This subtle difference allows it to be Sync even if T is not.
UnsyncIcTokenLock
Like IcTokenLock but requires presenting IcTokenUnsyncRef, which is Unsync. This subtle difference allows it to be Sync even if T is not.