pub struct ArcToken(/* private fields */);Available on crate feature
alloc and target_has_atomic="ptr" only.Expand description
An Arc-based unforgeable token used to access the contents of a
TokenLock.
This type lacks a Clone implementation to ensure exclusive access to
TokenLock.
Implementations
Sourceimpl ArcToken
impl ArcToken
pub fn new() -> Self
Sourcepub fn id(&self) -> ArcTokenId
pub fn id(&self) -> ArcTokenId
Construct an ArcTokenId that equates to self.
Sourcepub fn borrow_as_unsync(&mut self) -> ArcTokenUnsyncRef<'_>
pub fn borrow_as_unsync(&mut self) -> ArcTokenUnsyncRef<'_>
Borrow self as a non-Sync reference.
Non-Sync tokens such as those created by this method can be used with
UnsyncTokenLock.