Available on crate feature
std only.Expand description
Integer counter (IC) based tokens
This module provides a token type that is generated from a global 128-bit counter with thread-local pools to optimize for throughput. The runtime memory consumption is proportional to the maximum number of threads that existed simultaneously because the memory allocated to hold a freelist is never released.
Structs
- IcToken
- A counter-based unforgeable token used to access the contents of a
TokenLock. - IcToken
Id - Token that cannot be used to access the contents of a
TokenLock, but can be used to create a newTokenLock. - IcToken
Unsync Ref - Represents a borrow of
IcTokenconstrained to a single thread.
Type Aliases
- IcPin
Token Lock - A pinned mutual exclusive primitive that can be accessed by presenting the
matching
IcToken. - IcToken
Lock - A mutual exclusive primitive that can be accessed by presenting the matching
IcToken. - Unsync
IcPin Token Lock - Like
IcPinTokenLockbut requires presentingIcTokenUnsyncRef, which isUnsync. This subtle difference allows it to beSynceven ifTis not. - Unsync
IcToken Lock - Like
IcTokenLockbut requires presentingIcTokenUnsyncRef, which isUnsync. This subtle difference allows it to beSynceven ifTis not.